import json
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{fitzgerald2022massive,
      title={MASSIVE: A 1M-Example Multilingual Natural Language Understanding Dataset with 51 Typologically-Diverse Languages},
      author={Jack FitzGerald and Christopher Hench and Charith Peris and Scott Mackie and Kay Rottmann and Ana Sanchez and Aaron
      Nash and Liam Urbach and Vishesh Kakarala and Richa Singh and Swetha Ranganath and Laurie Crist and Misha Britan and Wouter
      Leeuwis and Gokhan Tur and Prem Natarajan},
      year={2022},
      eprint={2204.08582},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
@inproceedings{bastianelli-etal-2020-slurp,
    title = "{SLURP}: A Spoken Language Understanding Resource Package",
    author = "Bastianelli, Emanuele  and
      Vanzo, Andrea  and
      Swietojanski, Pawel  and
      Rieser, Verena",
    booktitle = "Proceedings of the 2020 Conference on Empirical Methods in Natural Language Processing (EMNLP)",
    month = nov,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2020.emnlp-main.588",
    doi = "10.18653/v1/2020.emnlp-main.588",
    pages = "7252--7262",
    abstract = "Spoken Language Understanding infers semantic meaning directly from audio data, and thus promises to
    reduce error propagation and misunderstandings in end-user applications. However, publicly available SLU resources are limited.
    In this paper, we release SLURP, a new SLU package containing the following: (1) A new challenging dataset in English spanning
    18 domains, which is substantially bigger and linguistically more diverse than existing datasets; (2) Competitive baselines
    based on state-of-the-art NLU and ASR systems; (3) A new transparent metric for entity labelling which enables a detailed error
    analysis for identifying potential areas of improvement. SLURP is available at https://github.com/pswietojanski/slurp."
}
"""
_DATASETNAME = "massive"
_DESCRIPTION = """\
MASSIVE dataset—Multilingual Amazon Slu resource package (SLURP) for Slot-filling, Intent classification, and
Virtual assistant Evaluation. MASSIVE contains 1M realistic, parallel, labeled virtual assistant utterances
spanning 18 domains, 60 intents, and 55 slots. MASSIVE was created by tasking professional translators to
localize the English-only SLURP dataset into 50 typologically diverse languages, including 8 native languages
and 2 other languages mostly spoken in Southeast Asia.
"""
_HOMEPAGE = "https://github.com/alexa/massive"
_LICENSE = Licenses.CC_BY_4_0.value
_LOCAL = False
_LANGUAGES = ["ind", "jav", "khm", "zlm", "mya", "tha", "tgl", "vie"]

_URLS = {
    _DATASETNAME: "https://amazon-massive-nlu-dataset.s3.amazonaws.com/amazon-massive-dataset-1.1.tar.gz",
}
_SUPPORTED_TASKS = [Tasks.INTENT_CLASSIFICATION, Tasks.SLOT_FILLING]
_SOURCE_VERSION = "1.1.0"
_SEACROWD_VERSION = "1.0.0"

# ind, jav, khm, zlm, mya, tha, tgl, vie, cmn, tam
_LANGS = [
    "af-ZA",
    "am-ET",
    "ar-SA",
    "az-AZ",
    "bn-BD",
    "cy-GB",
    "da-DK",
    "de-DE",
    "el-GR",
    "en-US",
    "es-ES",
    "fa-IR",
    "fi-FI",
    "fr-FR",
    "he-IL",
    "hi-IN",
    "hu-HU",
    "hy-AM",
    "id-ID",  # ind
    "is-IS",
    "it-IT",
    "ja-JP",
    "jv-ID",  # jav
    "ka-GE",
    "km-KH",  # khm
    "kn-IN",
    "ko-KR",
    "lv-LV",
    "ml-IN",
    "mn-MN",
    "ms-MY",  # zlm
    "my-MM",  # mya
    "nb-NO",
    "nl-NL",
    "pl-PL",
    "pt-PT",
    "ro-RO",
    "ru-RU",
    "sl-SL",
    "sq-AL",
    "sv-SE",
    "sw-KE",
    "ta-IN",
    "te-IN",
    "th-TH",  # tha
    "tl-PH",  # tgl
    "tr-TR",
    "ur-PK",
    "vi-VN",  # vie
    "zh-CN",  # cmn
    "zh-TW",
]
_SUBSETS = ["id-ID", "jv-ID", "km-KH", "ms-MY", "my-MM", "th-TH", "tl-PH", "vi-VN"]

_SCENARIOS = ["calendar", "recommendation", "social", "general", "news", "cooking", "iot", "email", "weather", "alarm", "transport", "lists", "takeaway", "play", "audio", "music", "qa", "datetime"]

_INTENTS = [
    "audio_volume_other",
    "play_music",
    "iot_hue_lighton",
    "general_greet",
    "calendar_set",
    "audio_volume_down",
    "social_query",
    "audio_volume_mute",
    "iot_wemo_on",
    "iot_hue_lightup",
    "audio_volume_up",
    "iot_coffee",
    "takeaway_query",
    "qa_maths",
    "play_game",
    "cooking_query",
    "iot_hue_lightdim",
    "iot_wemo_off",
    "music_settings",
    "weather_query",
    "news_query",
    "alarm_remove",
    "social_post",
    "recommendation_events",
    "transport_taxi",
    "takeaway_order",
    "music_query",
    "calendar_query",
    "lists_query",
    "qa_currency",
    "recommendation_movies",
    "general_joke",
    "recommendation_locations",
    "email_querycontact",
    "lists_remove",
    "play_audiobook",
    "email_addcontact",
    "lists_createoradd",
    "play_radio",
    "qa_stock",
    "alarm_query",
    "email_sendemail",
    "general_quirky",
    "music_likeness",
    "cooking_recipe",
    "email_query",
    "datetime_query",
    "transport_traffic",
    "play_podcasts",
    "iot_hue_lightchange",
    "calendar_remove",
    "transport_query",
    "transport_ticket",
    "qa_factoid",
    "iot_cleaning",
    "alarm_set",
    "datetime_convert",
    "iot_hue_lightoff",
    "qa_definition",
    "music_dislikeness",
]

_TAGS = [
    "O",
    "B-food_type",
    "B-movie_type",
    "B-person",
    "B-change_amount",
    "I-relation",
    "I-game_name",
    "B-date",
    "B-movie_name",
    "I-person",
    "I-place_name",
    "I-podcast_descriptor",
    "I-audiobook_name",
    "B-email_folder",
    "B-coffee_type",
    "B-app_name",
    "I-time",
    "I-coffee_type",
    "B-transport_agency",
    "B-podcast_descriptor",
    "I-playlist_name",
    "B-media_type",
    "B-song_name",
    "I-music_descriptor",
    "I-song_name",
    "B-event_name",
    "I-timeofday",
    "B-alarm_type",
    "B-cooking_type",
    "I-business_name",
    "I-color_type",
    "B-podcast_name",
    "I-personal_info",
    "B-weather_descriptor",
    "I-list_name",
    "B-transport_descriptor",
    "I-game_type",
    "I-date",
    "B-place_name",
    "B-color_type",
    "B-game_name",
    "I-artist_name",
    "I-drink_type",
    "B-business_name",
    "B-timeofday",
    "B-sport_type",
    "I-player_setting",
    "I-transport_agency",
    "B-game_type",
    "B-player_setting",
    "I-music_album",
    "I-event_name",
    "I-general_frequency",
    "I-podcast_name",
    "I-cooking_type",
    "I-radio_name",
    "I-joke_type",
    "I-meal_type",
    "I-transport_type",
    "B-joke_type",
    "B-time",
    "B-order_type",
    "B-business_type",
    "B-general_frequency",
    "I-food_type",
    "I-time_zone",
    "B-currency_name",
    "B-time_zone",
    "B-ingredient",
    "B-house_place",
    "B-audiobook_name",
    "I-ingredient",
    "I-media_type",
    "I-news_topic",
    "B-music_genre",
    "I-definition_word",
    "B-list_name",
    "B-playlist_name",
    "B-email_address",
    "I-currency_name",
    "I-movie_name",
    "I-device_type",
    "I-weather_descriptor",
    "B-audiobook_author",
    "I-audiobook_author",
    "I-app_name",
    "I-order_type",
    "I-transport_name",
    "B-radio_name",
    "I-business_type",
    "B-definition_word",
    "B-artist_name",
    "I-movie_type",
    "B-transport_name",
    "I-email_folder",
    "B-music_album",
    "I-house_place",
    "I-music_genre",
    "B-drink_type",
    "I-alarm_type",
    "B-music_descriptor",
    "B-news_topic",
    "B-meal_type",
    "I-transport_descriptor",
    "I-email_address",
    "I-change_amount",
    "B-device_type",
    "B-transport_type",
    "B-relation",
    "I-sport_type",
    "B-personal_info",
]


class MASSIVEDataset(datasets.GeneratorBasedBuilder):
    """MASSIVE datasets contains datasets to detect the intent from the text and fill the dialogue slots"""

    BUILDER_CONFIGS = (
        [
            SEACrowdConfig(
                name=f"massive_{subset}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"MASSIVE source schema for {subset}",
                schema="source",
                subset_id="massive_" + subset,
            )
            for subset in _SUBSETS
        ]
        + [
            SEACrowdConfig(
                name=f"massive_{subset}_seacrowd_text",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"MASSIVE Nusantara intent classification schema for {subset}",
                schema="seacrowd_text",
                subset_id="massive_intent_" + subset,
            )
            for subset in _SUBSETS
        ]
        + [
            SEACrowdConfig(
                name=f"massive_{subset}_seacrowd_seq_label",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"MASSIVE Nusantara slot filling schema for {subset}",
                schema="seacrowd_seq_label",
                subset_id="massive_slot_filling_" + subset,
            )
            for subset in _SUBSETS
        ]
        + [
            SEACrowdConfig(
                name="massive_source",
                version=datasets.Version(_SOURCE_VERSION),
                description="MASSIVE source schema",
                schema="source",
                subset_id="massive",
            ),
            SEACrowdConfig(
                name="massive_seacrowd_text",
                version=datasets.Version(_SEACROWD_VERSION),
                description="MASSIVE Nusantara intent classification schema",
                schema="seacrowd_text",
                subset_id="massive_intent",
            ),
            SEACrowdConfig(
                name="massive_seacrowd_seq_label",
                version=datasets.Version(_SEACROWD_VERSION),
                description="MASSIVE Nusantara slot filling schema",
                schema="seacrowd_seq_label",
                subset_id="massive_slot_filling",
            ),
        ]
    )

    DEFAULT_CONFIG_NAME = "massive_id-ID_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "locale": datasets.Value("string"),
                    "partition": datasets.Value("string"),
                    "scenario": datasets.features.ClassLabel(names=_SCENARIOS),
                    "intent": datasets.features.ClassLabel(names=_INTENTS),
                    "utt": datasets.Value("string"),
                    "annot_utt": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(datasets.features.ClassLabel(names=_TAGS)),
                    "worker_id": datasets.Value("string"),
                    "slot_method": datasets.Sequence(
                        {
                            "slot": datasets.Value("string"),
                            "method": datasets.Value("string"),
                        }
                    ),
                    "judgments": datasets.Sequence(
                        {
                            "worker_id": datasets.Value("string"),
                            "intent_score": datasets.Value("int8"),  # [0, 1, 2]
                            "slots_score": datasets.Value("int8"),  # [0, 1, 2]
                            "grammar_score": datasets.Value("int8"),  # [0, 1, 2, 3, 4]
                            "spelling_score": datasets.Value("int8"),  # [0, 1, 2]
                            "language_identification": datasets.Value("string"),
                        }
                    ),
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(label_names=_INTENTS)
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(label_names=_TAGS)
        else:
            raise ValueError(f"Invalid config schema: {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        archive = dl_manager.download(_URLS[_DATASETNAME])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "files": dl_manager.iter_archive(archive),
                    "split": "train",
                    "lang": self.config.name,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "files": dl_manager.iter_archive(archive),
                    "split": "dev",
                    "lang": self.config.name,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "files": dl_manager.iter_archive(archive),
                    "split": "test",
                    "lang": self.config.name,
                },
            ),
        ]

    def _get_bio_format(self, text):
        """This function is modified from https://huggingface.co/datasets/qanastek/MASSIVE/blob/main/MASSIVE.py"""
        tags, tokens = [], []

        bio_mode = False
        cpt_bio = 0
        current_tag = None

        split_iter = iter(text.split(" "))

        for s in split_iter:
            if s.startswith("["):
                current_tag = s.strip("[")
                bio_mode = True
                cpt_bio += 1
                next(split_iter)
                continue

            elif s.endswith("]"):
                bio_mode = False
                if cpt_bio == 1:
                    prefix = "B-"
                else:
                    prefix = "I-"
                token = prefix + current_tag
                word = s.strip("]")
                current_tag = None
                cpt_bio = 0

            else:
                if bio_mode:
                    if cpt_bio == 1:
                        prefix = "B-"
                    else:
                        prefix = "I-"
                    token = prefix + current_tag
                    word = s
                    cpt_bio += 1
                else:
                    token = "O"
                    word = s

            tags.append(token)
            tokens.append(word)

        return tokens, tags

    def _generate_examples(self, files: list, split: str, lang: str):
        _id = 0

        lang = lang.replace("massive_", "").replace("source", "").replace("seacrowd_text", "").replace("seacrowd_seq_label", "")

        if not lang:
            lang = _LANGS.copy()
        else:
            lang = [lang[:-1]]

        # logger.info("Generating examples from = %s", ", ".join(lang))

        for path, f in files:
            curr_lang = path.split(f"{_SOURCE_VERSION[:-2]}/data/")[-1].split(".jsonl")[0]

            if not lang:
                break
            elif curr_lang in lang:
                lang.remove(curr_lang)
            else:
                continue

            # Read the file
            lines = f.read().decode(encoding="utf-8").split("\n")

            for line in lines:
                data = json.loads(line)

                if data["partition"] != split:
                    continue

                # Slot method
                if "slot_method" in data:
                    slot_method = [
                        {
                            "slot": s["slot"],
                            "method": s["method"],
                        }
                        for s in data["slot_method"]
                    ]
                else:
                    slot_method = []

                # Judgments
                if "judgments" in data:
                    judgments = [
                        {
                            "worker_id": j["worker_id"],
                            "intent_score": j["intent_score"],
                            "slots_score": j["slots_score"],
                            "grammar_score": j["grammar_score"],
                            "spelling_score": j["spelling_score"],
                            "language_identification": j["language_identification"] if "language_identification" in j else "target",
                        }
                        for j in data["judgments"]
                    ]
                else:
                    judgments = []

                if self.config.schema == "source":
                    tokens, tags = self._get_bio_format(data["annot_utt"])

                    yield _id, {
                        "id": str(_id) + "_" + data["id"],
                        "locale": data["locale"],
                        "partition": data["partition"],
                        "scenario": data["scenario"],
                        "intent": data["intent"],
                        "utt": data["utt"],
                        "annot_utt": data["annot_utt"],
                        "tokens": tokens,
                        "ner_tags": tags,
                        "worker_id": data["worker_id"],
                        "slot_method": slot_method,
                        "judgments": judgments,
                    }

                elif self.config.schema == "seacrowd_seq_label":
                    tokens, tags = self._get_bio_format(data["annot_utt"])

                    yield _id, {
                        "id": str(_id) + "_" + data["id"],
                        "tokens": tokens,
                        "labels": tags,
                    }

                elif self.config.schema == "seacrowd_text":
                    yield _id, {
                        "id": str(_id) + "_" + data["id"],
                        "text": data["utt"],
                        "label": data["intent"],
                    }

                else:
                    raise ValueError(f"Invalid config: {self.config.name}")

                _id += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = """\
@misc{deng2023multilingual,
title={Multilingual Jailbreak Challenges in Large Language Models},
author={Yue Deng and Wenxuan Zhang and Sinno Jialin Pan and Lidong Bing},
year={2023},
eprint={2310.06474},
archivePrefix={arXiv},
primaryClass={cs.CL}
}
"""

_DATASETNAME = "xl_jailbreak"

_DESCRIPTION = """\
This dataset contains the data for the paper "Multilingual Jailbreak Challenges in Large Language Models".
"""

_HOMEPAGE = "https://huggingface.co/datasets/DAMO-NLP-SG/MultiJail"

_LANGUAGES = ["jav", "vie", "tha"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.MIT.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {"train": "https://huggingface.co/api/datasets/DAMO-NLP-SG/MultiJail/parquet/default/train/0.parquet"},
}

_SUPPORTED_TASKS = [Tasks.PROMPTING]
_SUPPORTED_SCHEMA_STRINGS = [f"seacrowd_{str(TASK_TO_SCHEMA[task]).lower()}" for task in _SUPPORTED_TASKS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LANGUAGE_TO_COLUMN = {
    "vie": "vi",
    "tha": "th",
    "jav": "jv",
}


@dataclass
class XlJailbreakSeacrowdConfig(SEACrowdConfig):
    """BuilderConfig for Nusantara."""

    language: str = None


class XlJailbreak(datasets.GeneratorBasedBuilder):
    """This dataset contains the data for the paper "Multilingual Jailbreak Challenges in Large Language Models"."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = []

    for language in _LANGUAGES:
        subset_id = language

        BUILDER_CONFIGS.append(
            XlJailbreakSeacrowdConfig(
                name=f"{subset_id}_source",
                version=SOURCE_VERSION,
                description=f"{_DATASETNAME} {language} schema",
                schema="source",
                subset_id=subset_id,
                language=language,
            )
        )

    seacrowd_schema_config: list[SEACrowdConfig] = []

    for seacrowd_schema in _SUPPORTED_SCHEMA_STRINGS:
        for language in _LANGUAGES:
            subset_id = language

            seacrowd_schema_config.append(
                XlJailbreakSeacrowdConfig(
                    name=f"{subset_id}_{seacrowd_schema}",
                    version=SEACROWD_VERSION,
                    description=f"{_DATASETNAME} {seacrowd_schema} schema",
                    schema=f"{seacrowd_schema}",
                    subset_id=subset_id,
                    language=language,
                )
            )

    BUILDER_CONFIGS.extend(seacrowd_schema_config)

    DEFAULT_CONFIG_NAME = f"{_LANGUAGES[0]}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value(dtype="int64"),
                    "source": datasets.Value(dtype="string"),
                    "tags": datasets.Value(dtype="string"),
                    "en": datasets.Value(dtype="string"),
                    "zh": datasets.Value(dtype="string"),
                    "it": datasets.Value(dtype="string"),
                    "vi": datasets.Value(dtype="string"),
                    "ar": datasets.Value(dtype="string"),
                    "ko": datasets.Value(dtype="string"),
                    "th": datasets.Value(dtype="string"),
                    "bn": datasets.Value(dtype="string"),
                    "sw": datasets.Value(dtype="string"),
                    "jv": datasets.Value(dtype="string"),
                }
            )

        elif self.config.schema == f"seacrowd_{str(TASK_TO_SCHEMA[Tasks.PROMPTING]).lower()}":
            features = schemas.ssp_features

        else:
            raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        train_path = dl_manager.download_and_extract(urls["train"])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":

            df = pd.read_parquet(filepath)

            for index, row in df.iterrows():
                yield index, row.to_dict()

        elif self.config.schema == f"seacrowd_{str(TASK_TO_SCHEMA[Tasks.PROMPTING]).lower()}":
            df = pd.read_parquet(filepath)

            # Apply the function to each row and create a new column with the JSON string
            df["text"] = df[_LANGUAGE_TO_COLUMN[self.config.language]]

            df = df[["id", "text"]]

            print(df)

            for index, row in df.iterrows():
                yield index, row.to_dict()

        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@article{mahadi2023indonesian,
    author    = {Made Raharja Surya Mahadi and Nugraha Priya Utama},
    title     = {Indonesian Text-to-Image Synthesis with Sentence-BERT and FastGAN},
    journal   = {arXiv preprint arXiv:2303.14517},
    year      = {2023},
    url       = {https://arxiv.org/abs/2303.14517},
}
"""

_DATASETNAME = "cub_bahasa"
_DESCRIPTION = """\
Semi-translated dataset of CUB-200-2011 into Indonesian. This dataset contains thousands
of image-text annotation pairs of 200 subcategories belonging to birds. The natural
language descriptions are collected through the Amazon Mechanical Turk (AMT) platform and
are required at least 10 words, without any information on subcategories and actions.
"""

_LOCAL=False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_HOMEPAGE = "https://github.com/share424/Indonesian-Text-to-Image-synthesis-with-Sentence-BERT-and-FastGAN"
_LICENSE = Licenses.UNKNOWN.value
_URLS = {
    "text": "https://raw.githubusercontent.com/share424/Indonesian-Text-to-Image-synthesis-with-Sentence-BERT-and-FastGAN/master/dataset/indo_cub_200_2011_captions.json",
    "image": "https://data.caltech.edu/records/65de6-vp158/files/CUB_200_2011.tgz"
}

_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class CubBahasaDataset(datasets.GeneratorBasedBuilder):
    """CUB-200-2011 image-text dataset in Indonesian language for bird domain."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "imtext"
    IMAGE_CLASS = {
        1: '001.Black_footed_Albatross',
        2: '002.Laysan_Albatross',
        3: '003.Sooty_Albatross',
        4: '004.Groove_billed_Ani',
        5: '005.Crested_Auklet',
        6: '006.Least_Auklet',
        7: '007.Parakeet_Auklet',
        8: '008.Rhinoceros_Auklet',
        9: '009.Brewer_Blackbird',
        10: '010.Red_winged_Blackbird',
        11: '011.Rusty_Blackbird',
        12: '012.Yellow_headed_Blackbird',
        13: '013.Bobolink',
        14: '014.Indigo_Bunting',
        15: '015.Lazuli_Bunting',
        16: '016.Painted_Bunting',
        17: '017.Cardinal',
        18: '018.Spotted_Catbird',
        19: '019.Gray_Catbird',
        20: '020.Yellow_breasted_Chat',
        21: '021.Eastern_Towhee',
        22: '022.Chuck_will_Widow',
        23: '023.Brandt_Cormorant',
        24: '024.Red_faced_Cormorant',
        25: '025.Pelagic_Cormorant',
        26: '026.Bronzed_Cowbird',
        27: '027.Shiny_Cowbird',
        28: '028.Brown_Creeper',
        29: '029.American_Crow',
        30: '030.Fish_Crow',
        31: '031.Black_billed_Cuckoo',
        32: '032.Mangrove_Cuckoo',
        33: '033.Yellow_billed_Cuckoo',
        34: '034.Gray_crowned_Rosy_Finch',
        35: '035.Purple_Finch',
        36: '036.Northern_Flicker',
        37: '037.Acadian_Flycatcher',
        38: '038.Great_Crested_Flycatcher',
        39: '039.Least_Flycatcher',
        40: '040.Olive_sided_Flycatcher',
        41: '041.Scissor_tailed_Flycatcher',
        42: '042.Vermilion_Flycatcher',
        43: '043.Yellow_bellied_Flycatcher',
        44: '044.Frigatebird',
        45: '045.Northern_Fulmar',
        46: '046.Gadwall',
        47: '047.American_Goldfinch',
        48: '048.European_Goldfinch',
        49: '049.Boat_tailed_Grackle',
        50: '050.Eared_Grebe',
        51: '051.Horned_Grebe',
        52: '052.Pied_billed_Grebe',
        53: '053.Western_Grebe',
        54: '054.Blue_Grosbeak',
        55: '055.Evening_Grosbeak',
        56: '056.Pine_Grosbeak',
        57: '057.Rose_breasted_Grosbeak',
        58: '058.Pigeon_Guillemot',
        59: '059.California_Gull',
        60: '060.Glaucous_winged_Gull',
        61: '061.Heermann_Gull',
        62: '062.Herring_Gull',
        63: '063.Ivory_Gull',
        64: '064.Ring_billed_Gull',
        65: '065.Slaty_backed_Gull',
        66: '066.Western_Gull',
        67: '067.Anna_Hummingbird',
        68: '068.Ruby_throated_Hummingbird',
        69: '069.Rufous_Hummingbird',
        70: '070.Green_Violetear',
        71: '071.Long_tailed_Jaeger',
        72: '072.Pomarine_Jaeger',
        73: '073.Blue_Jay',
        74: '074.Florida_Jay',
        75: '075.Green_Jay',
        76: '076.Dark_eyed_Junco',
        77: '077.Tropical_Kingbird',
        78: '078.Gray_Kingbird',
        79: '079.Belted_Kingfisher',
        80: '080.Green_Kingfisher',
        81: '081.Pied_Kingfisher',
        82: '082.Ringed_Kingfisher',
        83: '083.White_breasted_Kingfisher',
        84: '084.Red_legged_Kittiwake',
        85: '085.Horned_Lark',
        86: '086.Pacific_Loon',
        87: '087.Mallard',
        88: '088.Western_Meadowlark',
        89: '089.Hooded_Merganser',
        90: '090.Red_breasted_Merganser',
        91: '091.Mockingbird',
        92: '092.Nighthawk',
        93: '093.Clark_Nutcracker',
        94: '094.White_breasted_Nuthatch',
        95: '095.Baltimore_Oriole',
        96: '096.Hooded_Oriole',
        97: '097.Orchard_Oriole',
        98: '098.Scott_Oriole',
        99: '099.Ovenbird',
        100: '100.Brown_Pelican',
        101: '101.White_Pelican',
        102: '102.Western_Wood_Pewee',
        103: '103.Sayornis',
        104: '104.American_Pipit',
        105: '105.Whip_poor_Will',
        106: '106.Horned_Puffin',
        107: '107.Common_Raven',
        108: '108.White_necked_Raven',
        109: '109.American_Redstart',
        110: '110.Geococcyx',
        111: '111.Loggerhead_Shrike',
        112: '112.Great_Grey_Shrike',
        113: '113.Baird_Sparrow',
        114: '114.Black_throated_Sparrow',
        115: '115.Brewer_Sparrow',
        116: '116.Chipping_Sparrow',
        117: '117.Clay_colored_Sparrow',
        118: '118.House_Sparrow',
        119: '119.Field_Sparrow',
        120: '120.Fox_Sparrow',
        121: '121.Grasshopper_Sparrow',
        122: '122.Harris_Sparrow',
        123: '123.Henslow_Sparrow',
        124: '124.Le_Conte_Sparrow',
        125: '125.Lincoln_Sparrow',
        126: '126.Nelson_Sharp_tailed_Sparrow',
        127: '127.Savannah_Sparrow',
        128: '128.Seaside_Sparrow',
        129: '129.Song_Sparrow',
        130: '130.Tree_Sparrow',
        131: '131.Vesper_Sparrow',
        132: '132.White_crowned_Sparrow',
        133: '133.White_throated_Sparrow',
        134: '134.Cape_Glossy_Starling',
        135: '135.Bank_Swallow',
        136: '136.Barn_Swallow',
        137: '137.Cliff_Swallow',
        138: '138.Tree_Swallow',
        139: '139.Scarlet_Tanager',
        140: '140.Summer_Tanager',
        141: '141.Artic_Tern',
        142: '142.Black_Tern',
        143: '143.Caspian_Tern',
        144: '144.Common_Tern',
        145: '145.Elegant_Tern',
        146: '146.Forsters_Tern',
        147: '147.Least_Tern',
        148: '148.Green_tailed_Towhee',
        149: '149.Brown_Thrasher',
        150: '150.Sage_Thrasher',
        151: '151.Black_capped_Vireo',
        152: '152.Blue_headed_Vireo',
        153: '153.Philadelphia_Vireo',
        154: '154.Red_eyed_Vireo',
        155: '155.Warbling_Vireo',
        156: '156.White_eyed_Vireo',
        157: '157.Yellow_throated_Vireo',
        158: '158.Bay_breasted_Warbler',
        159: '159.Black_and_white_Warbler',
        160: '160.Black_throated_Blue_Warbler',
        161: '161.Blue_winged_Warbler',
        162: '162.Canada_Warbler',
        163: '163.Cape_May_Warbler',
        164: '164.Cerulean_Warbler',
        165: '165.Chestnut_sided_Warbler',
        166: '166.Golden_winged_Warbler',
        167: '167.Hooded_Warbler',
        168: '168.Kentucky_Warbler',
        169: '169.Magnolia_Warbler',
        170: '170.Mourning_Warbler',
        171: '171.Myrtle_Warbler',
        172: '172.Nashville_Warbler',
        173: '173.Orange_crowned_Warbler',
        174: '174.Palm_Warbler',
        175: '175.Pine_Warbler',
        176: '176.Prairie_Warbler',
        177: '177.Prothonotary_Warbler',
        178: '178.Swainson_Warbler',
        179: '179.Tennessee_Warbler',
        180: '180.Wilson_Warbler',
        181: '181.Worm_eating_Warbler',
        182: '182.Yellow_Warbler',
        183: '183.Northern_Waterthrush',
        184: '184.Louisiana_Waterthrush',
        185: '185.Bohemian_Waxwing',
        186: '186.Cedar_Waxwing',
        187: '187.American_Three_toed_Woodpecker',
        188: '188.Pileated_Woodpecker',
        189: '189.Red_bellied_Woodpecker',
        190: '190.Red_cockaded_Woodpecker',
        191: '191.Red_headed_Woodpecker',
        192: '192.Downy_Woodpecker',
        193: '193.Bewick_Wren',
        194: '194.Cactus_Wren',
        195: '195.Carolina_Wren',
        196: '196.House_Wren',
        197: '197.Marsh_Wren',
        198: '198.Rock_Wren',
        199: '199.Winter_Wren',
        200: '200.Common_Yellowthroat'
    }

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "image_id": datasets.Value("int64"),
                    "class_id": datasets.Value("int64"),
                    "image_path": datasets.Value("string"),
                    "class_name": datasets.Value("string"),
                    "captions": [
                        {
                            "caption_eng": datasets.Value("string"),
                            "caption_ind": datasets.Value("string"),
                        }
                    ]
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.image_text_features(label_names=list(self.IMAGE_CLASS.values()))

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        # expect several minutes to download image data ~1.2GB
        data_path = dl_manager.download_and_extract(_URLS)

        # working with image dataset
        image_meta = Path(data_path["image"]) / "CUB_200_2011" / "images.txt"
        df_image = pd.read_csv(image_meta, sep=" ", names=["image_id", "image_path"])
        df_image['image_path'] = df_image['image_path'].apply(lambda x: Path(image_meta.parent, 'images', x))

        label_meta = Path(data_path["image"]) / "CUB_200_2011" / "image_class_labels.txt"
        df_label = pd.read_csv(label_meta, sep=" ", names=["image_id", "class_id"])

        # working with text dataset
        text_path = Path(data_path["text"])
        with open(text_path, "r") as f:
            text_data = json.load(f)

        df_text = pd.DataFrame([
            {
                'image_name': item['filename'],
                'en_caption': caption['english'],
                'id_caption': caption['indo']
            } for item in text_data['dataset'] for caption in item['captions']
        ])
        grouped_text = df_text.groupby('image_name').agg(list).reset_index()

        # working with split
        split_dir = Path(data_path["image"]) / "CUB_200_2011" / "train_test_split.txt"
        df_split = pd.read_csv(split_dir, sep=" ", names=["image_id", "is_train"])

        # merge all data
        df_image['image_name'] = df_image['image_path'].apply(lambda x: x.name)
        df = pd.merge(df_image, grouped_text, on="image_name")
        df.drop(columns=['image_name'], inplace=True)

        df = pd.merge(df, df_label, on="image_id")
        df = pd.merge(df, df_split, on="image_id")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "data": df[df['is_train'] == 1],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "data": df[df['is_train'] == 0],
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, data: pd.DataFrame, split: str) -> Tuple[int, Dict]:
        if self.config.schema == "source":
            for key, row in data.iterrows():
                example = {
                    "image_id": row["image_id"],
                    "class_id": row["class_id"],
                    "image_path": row["image_path"],
                    "class_name": self.IMAGE_CLASS[row["class_id"]],
                    "captions": [
                        {
                            "caption_eng": row["en_caption"][i],
                            "caption_ind": row["id_caption"][i],
                        } for i in range(len(row["en_caption"]))
                    ]
                }
                yield key, example
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            key = 0
            for _, row in data.iterrows():
                for i in range(len(row["id_caption"])):
                    example = {
                        "id": str(key),
                        "image_paths": [row["image_path"]],
                        "texts": row["id_caption"][i],
                        "metadata": {
                            "context": row["en_caption"][i],
                            "labels": [self.IMAGE_CLASS[row["class_id"]]],
                        }
                    }
                    yield key, example
                    key += 1


from pathlib import Path
from typing import List

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import DEFAULT_SEACROWD_VIEW_NAME, DEFAULT_SOURCE_VIEW_NAME, Tasks

_DATASETNAME = "emot"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 langauge code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@inproceedings{saputri2018emotion,
  title={Emotion classification on indonesian twitter dataset},
  author={Saputri, Mei Silviana and Mahendra, Rahmad and Adriani, Mirna},
  booktitle={2018 International Conference on Asian Language Processing (IALP)},
  pages={90--95},
  year={2018},
  organization={IEEE}
}

@inproceedings{wilie2020indonlu,
  title={IndoNLU: Benchmark and Resources for Evaluating Indonesian Natural Language Understanding},
  author={Wilie, Bryan and Vincentio, Karissa and Winata, Genta Indra and Cahyawijaya, Samuel and Li, Xiaohong and Lim, Zhi Yuan and Soleman, Sidik and Mahendra, Rahmad and Fung, Pascale and Bahar, Syafri and others},
  booktitle={Proceedings of the 1st Conference of the Asia-Pacific Chapter of the Association for Computational Linguistics and the 10th International Joint Conference on Natural Language Processing},
  pages={843--857},
  year={2020}
}
"""

_DESCRIPTION = """\
EmoT is an emotion classification dataset collected from the social media platform Twitter. The dataset consists of around 4000 Indonesian colloquial language tweets, covering five different emotion labels: anger, fear, happiness, love, and sadness.
EmoT dataset is splitted into 3 sets with 3521 train, 440 validation, 442 test data.
"""

_HOMEPAGE = "https://github.com/IndoNLP/indonlu"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_URLs = {
    "train": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/emot_emotion-twitter/train_preprocess.csv",
    "validation": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/emot_emotion-twitter/valid_preprocess.csv",
    "test": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/emot_emotion-twitter/test_preprocess.csv",
}

_SUPPORTED_TASKS = [Tasks.EMOTION_CLASSIFICATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class EmoT(datasets.GeneratorBasedBuilder):
    """SMSA is a sentiment analysis dataset consisting of 3 labels (positive, neutral, and negative) which comes from comments and reviews collected from multiple online platforms."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="emot_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="EmoT source schema",
            schema="source",
            subset_id="emot",
        ),
        SEACrowdConfig(
            name="emot_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description="EmoT Nusantara schema",
            schema="seacrowd_text",
            subset_id="emot",
        ),
    ]

    DEFAULT_CONFIG_NAME = "emot_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("string"),
                    "tweet": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["happy", "love", "fear", "anger", "sadness"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        train_tsv_path = Path(dl_manager.download_and_extract(_URLs["train"]))
        validation_tsv_path = Path(dl_manager.download_and_extract(_URLs["validation"]))
        test_tsv_path = Path(dl_manager.download_and_extract(_URLs["test"]))
        data_files = {
            "train": train_tsv_path,
            "validation": validation_tsv_path,
            "test": test_tsv_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        df = pd.read_csv(filepath, sep=",", header="infer").reset_index()
        df.columns = ["id", "label", "tweet"]

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {"index": str(row.id), "tweet": row.tweet, "label": row.label}
                yield row.id, ex
        elif self.config.schema == "seacrowd_text":
            for row in df.itertuples():
                ex = {"id": str(row.id), "text": row.tweet, "label": row.label}
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import csv
import os
from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)

_DATASETNAME = "su_id_tts"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["sun"]
_LOCAL = False
_CITATION = """\
@inproceedings{sodimana18_sltu,
  author={Keshan Sodimana and Pasindu {De Silva} and Supheakmungkol Sarin and Oddur Kjartansson and Martin Jansche and Knot Pipatsrisawat and Linne Ha},
  title={{A Step-by-Step Process for Building TTS Voices Using Open Source Data and Frameworks for Bangla, Javanese, Khmer, Nepali, Sinhala, and Sundanese}},
  year=2018,
  booktitle={Proc. 6th Workshop on Spoken Language Technologies for Under-Resourced Languages (SLTU 2018)},
  pages={66--70},
  doi={10.21437/SLTU.2018-14}
}
"""

_DESCRIPTION = """\
This data set contains high-quality transcribed audio data for Sundanese. The data set consists of wave files, and a TSV file. The file line_index.tsv contains a filename and the transcription of audio in the file. Each filename is prepended with a speaker identification number.
The data set has been manually quality checked, but there might still be errors.
This dataset was collected by Google in collaboration with Universitas Pendidikan Indonesia.
"""

_HOMEPAGE = "http://openslr.org/44/"

_LICENSE = "CC BY-SA 4.0"

_URLs = {
    _DATASETNAME: {
        "female": "https://www.openslr.org/resources/44/su_id_female.zip",
        "male": "https://www.openslr.org/resources/44/su_id_male.zip",
    }
}

_SUPPORTED_TASKS = [Tasks.TEXT_TO_SPEECH]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class SuIdTTS(datasets.GeneratorBasedBuilder):
    """su_id_tts contains high-quality Multi-speaker TTS data for Sundanese (SU-ID)."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="su_id_tts_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="SU_ID_TTS source schema",
            schema="source",
            subset_id="su_id_tts",
        ),
        SEACrowdConfig(
            name="su_id_tts_seacrowd_sptext",
            version=datasets.Version(_SEACROWD_VERSION),
            description="SU_ID_TTS Nusantara schema",
            schema="seacrowd_sptext",
            subset_id="su_id_tts",
        ),
    ]

    DEFAULT_CONFIG_NAME = "su_id_tts_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                    "gender": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[datasets.AutomaticSpeechRecognition(audio_column="audio", transcription_column="text")],
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        male_path = Path(dl_manager.download_and_extract(_URLs[_DATASETNAME]["male"]))
        female_path = Path(dl_manager.download_and_extract(_URLs[_DATASETNAME]["female"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "male_filepath": male_path,
                    "female_filepath": female_path,
                },
            ),
        ]

    def _generate_examples(self, male_filepath: Path, female_filepath: Path):

        if self.config.schema == "source" or self.config.schema == "seacrowd_sptext":
            tsv_m = os.path.join(male_filepath, "su_id_male", "line_index.tsv")
            tsv_f = os.path.join(female_filepath, "su_id_female", "line_index.tsv")

            with open(tsv_m, "r") as file:
                tsv_m_data = csv.reader(file, delimiter="\t")
                for line in tsv_m_data:
                    spk_trans_info = line[0].split("_")
                    if self.config.schema == "source":
                        ex = {
                            "id": line[0],
                            "speaker_id": spk_trans_info[0] + "_" + spk_trans_info[1],
                            "path": os.path.join(male_filepath, "su_id_male", "wavs", "{}.wav".format(line[0])),
                            "audio": os.path.join(male_filepath, "su_id_male", "wavs", "{}.wav".format(line[0])),
                            "text": line[2],
                            "gender": spk_trans_info[0][2],
                        }
                        yield line[0], ex

                    elif self.config.schema == "seacrowd_sptext":
                        ex = {
                            "id": line[0],
                            "speaker_id": spk_trans_info[0] + "_" + spk_trans_info[1],
                            "path": os.path.join(male_filepath, "su_id_male", "wavs", "{}.wav".format(line[0])),
                            "audio": os.path.join(male_filepath, "su_id_male", "wavs", "{}.wav".format(line[0])),
                            "text": line[2],
                            "metadata": {
                                "speaker_age": None,
                                "speaker_gender": spk_trans_info[0][2],
                            },
                        }
                        yield line[0], ex

            with open(tsv_f, "r") as file:
                tsv_f_data = csv.reader(file, delimiter="\t")
                for line in tsv_f_data:
                    spk_trans_info = line[0].split("_")
                    if self.config.schema == "source":
                        ex = {
                            "id": line[0],
                            "speaker_id": spk_trans_info[0] + "_" + spk_trans_info[1],
                            "path": os.path.join(female_filepath, "su_id_female", "wavs", "{}.wav".format(line[0])),
                            "audio": os.path.join(female_filepath, "su_id_female", "wavs", "{}.wav".format(line[0])),
                            "text": line[2],
                            "gender": spk_trans_info[0][2],
                        }
                        yield line[0], ex

                    elif self.config.schema == "seacrowd_sptext":
                        ex = {
                            "id": line[0],
                            "speaker_id": spk_trans_info[0] + "_" + spk_trans_info[1],
                            "path": os.path.join(female_filepath, "su_id_female", "wavs", "{}.wav".format(line[0])),
                            "audio": os.path.join(female_filepath, "su_id_female", "wavs", "{}.wav".format(line[0])),
                            "text": line[2],
                            "metadata": {
                                "speaker_age": None,
                                "speaker_gender": spk_trans_info[0][2],
                            },
                        }
                        yield line[0], ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
import csv
import json
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{commonvoice:2020,
  author = {Ardila, R. and Branson, M. and Davis, K. and Henretty, M. and Kohler, M. and Meyer, J. and Morais, R. and Saunders, L. and Tyers, F. M. and Weber, G.},
  title = {Common Voice: A Massively-Multilingual Speech Corpus},
  booktitle = {Proceedings of the 12th Conference on Language Resources and Evaluation (LREC 2020)},
  pages = {4211--4215},
  year = 2020
}
"""

_DATASETNAME = "commonvoice_120"

_DESCRIPTION = """\
The Common Mozilla Voice dataset consists of a unique MP3 and corresponding text file.
Many of the 26119 recorded hours in the dataset also include demographic metadata like age, sex, and accent that can help improve the accuracy of speech recognition engines.
The dataset currently consists of 17127 validated hours in 104 languages, but more voices and languages are always added.

Before using this dataloader, please accept the acknowledgement at https://huggingface.co/datasets/mozilla-foundation/common_voice_12_0 and use huggingface-cli login for authentication
"""

_HOMEPAGE = "https://commonvoice.mozilla.org/en/datasets"

_LANGUAGES = ["cnh", "ind", "tha", "vie"]
_LANG_TO_CVLANG = {"cnh": "cnh", "ind": "id", "tha": "th", "vie": "vi"}

_AGE_TO_INT = {"": None, "teens": 10, "twenties": 20, "thirties": 30, "fourties": 40, "fifties": 50, "sixties": 60, "seventies": 70, "eighties": 80}

_LICENSE = Licenses.CC0_1_0.value

# Note: the dataset is gated in HuggingFace. It's public after providing access token
_LOCAL = False

_COMMONVOICE_URL_TEMPLATE = "https://huggingface.co/datasets/mozilla-foundation/common_voice_12_0/resolve/main/"
_URLS = {"audio": _COMMONVOICE_URL_TEMPLATE + "audio/{lang}/{split}/{lang}_{split}_{shard_idx}.tar", "transcript": _COMMONVOICE_URL_TEMPLATE + "transcript/{lang}/{split}.tsv", "n_shards": _COMMONVOICE_URL_TEMPLATE + "n_shards.json"}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION, Tasks.TEXT_TO_SPEECH]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class Commonvoice120(datasets.GeneratorBasedBuilder):
    """This is the dataloader for CommonVoice 12.0 Mozilla"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        *[
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{lang}{'_' if lang else ''}source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"{_DATASETNAME} source schema for {lang}",
                schema="source",
                subset_id=f"{_DATASETNAME}{'_' if lang else ''}{lang}",
            )
            for lang in ["", *_LANGUAGES]
        ],
        *[
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{lang}{'_' if lang else ''}seacrowd_sptext",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"{_DATASETNAME} SEACrowd schema for {lang}",
                schema="seacrowd_sptext",
                subset_id=f"{_DATASETNAME}{'_' if lang else ''}{lang}",
            )
            for lang in ["", *_LANGUAGES]
        ],
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "client_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.features.Audio(sampling_rate=48_000),
                    "sentence": datasets.Value("string"),
                    "up_votes": datasets.Value("int64"),
                    "down_votes": datasets.Value("int64"),
                    "age": datasets.Value("string"),
                    "gender": datasets.Value("string"),
                    "accent": datasets.Value("string"),
                    "locale": datasets.Value("string"),
                    "segment": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        lang_code = self.config.subset_id.split("_")[-1]
        languages = [_LANG_TO_CVLANG.get(lang, lang) for lang in (_LANGUAGES if lang_code == "120" else [lang_code])]
        n_shards_path = dl_manager.download_and_extract(_URLS["n_shards"])
        with open(n_shards_path, encoding="utf-8") as f:
            n_shards = json.load(f)

        audio_urls = {}
        meta_urls = {}
        splits = ("train", "dev", "test")
        for split in splits:
            audio_urls[split] = [_URLS["audio"].format(lang=lang, split=split, shard_idx=i) for lang in languages for i in range(n_shards[lang][split])]
            meta_urls[split] = [_URLS["transcript"].format(lang=lang, split=split) for lang in languages]
        archive_paths = dl_manager.download(audio_urls)
        local_extracted_archive_paths = dl_manager.extract(archive_paths)
        meta_paths = dl_manager.download_and_extract(meta_urls)

        split_names = {
            "train": datasets.Split.TRAIN,
            "dev": datasets.Split.VALIDATION,
            "test": datasets.Split.TEST,
        }
        return [
            datasets.SplitGenerator(
                name=split_names.get(split, split),
                gen_kwargs={
                    "local_extracted_archive_paths": local_extracted_archive_paths.get(split),
                    "audio_archives": [dl_manager.iter_archive(path) for path in archive_paths.get(split)],
                    "meta_paths": meta_paths[split],
                    "split": "train",
                },
            )
            for split in splits
        ]

    def _generate_examples(self, local_extracted_archive_paths: [Path], audio_archives: [Path], meta_paths: [Path], split: str) -> Tuple[int, Dict]:
        data_fields = list(self._info().features.keys())
        metadata = {}
        for meta_path in meta_paths:
            with open(meta_path, encoding="utf-8") as f:
                reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
                for row in reader:
                    if not row["path"].endswith(".mp3"):
                        row["path"] += ".mp3"
                    if "accents" in row:
                        row["accent"] = row["accents"]
                        del row["accents"]
                    for field in data_fields:
                        if field not in row:
                            row[field] = ""
                    metadata[row["path"]] = row

        if self.config.schema == "source":
            for i, audio_archive in enumerate(audio_archives):
                for path, file in audio_archive:
                    _, filename = os.path.split(path)
                    if filename in metadata:
                        src_result = dict(metadata[filename])
                        path = os.path.join(local_extracted_archive_paths[i], path)
                        result = {
                            "client_id": src_result["client_id"],
                            "path": path,
                            "audio": {"path": path, "bytes": file.read()},
                            "sentence": src_result["sentence"],
                            "up_votes": src_result["up_votes"],
                            "down_votes": src_result["down_votes"],
                            "age": src_result["age"],
                            "gender": src_result["gender"],
                            "accent": src_result["accent"],
                            "locale": src_result["locale"],
                            "segment": src_result["segment"],
                        }
                        yield path, result

        elif self.config.schema == "seacrowd_sptext":
            for i, audio_archive in enumerate(audio_archives):
                for path, file in audio_archive:
                    _, filename = os.path.split(path)
                    if filename in metadata:
                        src_result = dict(metadata[filename])
                        # set the audio feature and the path to the extracted file
                        path = os.path.join(local_extracted_archive_paths[i], path)
                        result = {
                            "id": src_result["path"].replace(".mp3", ""),
                            "path": path,
                            "audio": {"path": path, "bytes": file.read()},
                            "text": src_result["sentence"],
                            "speaker_id": src_result["client_id"],
                            "metadata": {
                                "speaker_age": _AGE_TO_INT[src_result["age"]],
                                "speaker_gender": src_result["gender"],
                            },
                        }
                        yield path, result



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@article{riccosan2023,
  author    = {Riccosan and Saputra, Karen Etania},
  title     = {Multilabel multiclass sentiment and emotion dataset from indonesian mobile application review},
  journal   = {Data in Brief},
  volume    = {50},
  year      = {2023},
  doi       = {10.1016/j.dib.2023.109576},
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]
_DATASETNAME = "id_sent_emo_mobile_apps"
_DESCRIPTION = """
This dataset contains manually annotated public reviews of mobile applications in Indonesia.
Each review is given a sentiment label (positive, negative, neutral) and
an emotion label (anger, sadness, fear, happiness, love, neutral).
"""
_HOMEPAGE = "https://github.com/Ricco48/Multilabel-Sentiment-and-Emotion-Dataset-from-Indonesian-" "Mobile-Application-Review/tree/CreateCodeForPaper"
_LICENSE = Licenses.CC_BY_NC_ND_4_0.value
_URL = (
    "https://github.com/Ricco48/Multilabel-Sentiment-and-Emotion-Dataset-from-Indonesian-Mobile-Application-Review/raw/CreateCodeForPaper/"
    "Multilabel%20Sentiment%20and%20Emotion%20Dataset%20from%20Indonesian%20Mobile%20Application%20Review/Multilabel%20Sentiment%20and%20Emotion"
    "%20Dataset%20from%20Indonesian%20Mobile%20Application%20Review.csv"
)

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS, Tasks.EMOTION_CLASSIFICATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class EmoSentIndMobile(datasets.GeneratorBasedBuilder):
    """Dataset of Indonesian mobile application reviews manually annotated for emotion and sentiment."""

    SUBSETS = ["emotion", "sentiment"]
    EMOTION_CLASS_LABELS = ["Sad", "Anger", "Fear", "Happy", "Love", "Neutral"]
    SENTIMENT_CLASS_LABELS = ["Negative", "Positive", "Neutral"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME
        )
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema for {subset} subset",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}_{subset}",
        )
        for subset in SUBSETS
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "content": datasets.Value("string"),
                    "sentiment": datasets.Value("string"),
                    "emotion": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_text":
            if "emotion" in self.config.subset_id:
                labels = self.EMOTION_CLASS_LABELS
            elif "sentiment" in self.config.subset_id:
                labels = self.SENTIMENT_CLASS_LABELS
            features = schemas.text_features(label_names=labels)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        fp = dl_manager.download(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": fp},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        df = pd.read_csv(filepath, sep="\t", index_col=None)
        for index, row in df.iterrows():
            if self.config.schema == "source":
                example = {
                    "content": row["content"],
                    "sentiment": row["Sentiment"].title(),
                    "emotion": row["Emotion"].title(),
                }
            elif self.config.schema == "seacrowd_text":
                if "emotion" in self.config.subset_id:
                    label = row["Emotion"]
                elif "sentiment" in self.config.subset_id:
                    label = row["Sentiment"]
                example = {"id": str(index), "text": row["content"], "label": label.title()}
            yield index, example



from pathlib import Path

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks


_CITATION = """
@misc{myXNLI2023,
    title = "myXNLI",
    author = "akhtet",
    year = "202",
    url = "https://github.com/akhtet/myXNLI",
}
"""

_DATASETNAME = "myxnli"

_DESCRIPTION = """
The myXNLI corpus is a collection of Myanmar language data designed for the Natural Language Inference (NLI) task, which
originated from the XNLI and MultiNLI English datasets. The 7,500 sentence pairs from the XNLI English development and
test sets are human-translated into Myanmar. The 392,702 data from the NLI English training data is translated using
machine translation. In addition, it also extends its scope by adding Myanmar translations to the XNLI 15-language
parallel corpus, to create a 16-language parallel corpus.
"""

_HOMEPAGE = "https://github.com/akhtet/myXNLI"

_LANGUAGES = ["mya"]

_LICENSE = Licenses.CC_BY_NC_4_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {
        "train": "https://huggingface.co/datasets/akhtet/myXNLI/resolve/main/data/train-00000-of-00001-2614419e00195781.parquet",
        "dev": "https://huggingface.co/datasets/akhtet/myXNLI/resolve/main/data/validation-00000-of-00001-9c168eb31d1d810b.parquet",
        "test": "https://huggingface.co/datasets/akhtet/myXNLI/resolve/main/data/test-00000-of-00001-0fd9f93baf8c9cdb.parquet",
    },
}

_SUPPORTED_TASKS = [Tasks.TEXTUAL_ENTAILMENT]

_SOURCE_VERSION = "1.1.0"

_SEACROWD_VERSION = "1.0.0"


class MyXNLIDataset(datasets.GeneratorBasedBuilder):
    """The myXNLI corpus is a collection of Myanmar language data designed for the Natural Language Inference task."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_pairs",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_pairs",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "genre": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=["contradiction", "entailment", "neutral"]),
                    "sentence1_en": datasets.Value("string"),
                    "sentence2_en": datasets.Value("string"),
                    "sentence1_my": datasets.Value("string"),
                    "sentence2_my": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_pairs":
            features = schemas.pairs_features(["contradiction", "entailment", "neutral"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> list[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_dir, "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_dir, "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_dir, "split": "dev"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> tuple[int, dict]:
        if self.config.schema == "source":
            df = pd.read_parquet(filepath[split])
            for i, row in df.iterrows():
                yield i, {
                    "genre": row["genre"],
                    "label": row["label"],
                    "sentence1_en": row["sentence1_en"],
                    "sentence2_en": row["sentence2_en"],
                    "sentence1_my": row["sentence1_my"],
                    "sentence2_my": row["sentence2_my"],
                }

        elif self.config.schema == "seacrowd_pairs":
            df = pd.read_parquet(filepath[split])
            for i, row in df.iterrows():
                yield i, {
                    "id": str(i),
                    "text_1": row["sentence1_my"],
                    "text_2": row["sentence2_my"],
                    "label": row["label"],
                }



import datasets
import pandas

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                      DEFAULT_SOURCE_VIEW_NAME, Tasks, Licenses)

_DATASETNAME = "id_vaccines_tweets"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@article{febriyanti2021analisis,
  title={ANALISIS SENTIMEN MASYARAKAT INDONESIA TERHADAP PELAKSANAAN VAKSIN COVID'19},
  author={Febriyanti, Syintya and Nursidah, Dea Ratu and Gustiara, Dela and Yulianti, Rika},
  journal={Khazanah: Jurnal Mahasiswa},
  volume={13},
  number={2},
  year={2021}
}
"""

_DESCRIPTION = """\
Dataset containing tweets about COVID-19 vaccines with manually labelled information about whether they are a
subjective tweet and their sentiment polarity. Tweets are from 20-27 June 2021 and 15-22 July 2021.
"""

_HOMEPAGE = "https://github.com/rayendito/id-vaccines-tweets"

_LICENSE = Licenses.UNKNOWN.value

_URL = "https://raw.githubusercontent.com/rayendito/id-vaccines-tweets/main/id_vaccines_tweets.csv"

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IdVaccinesTweetsDataset(datasets.GeneratorBasedBuilder):
    """This is a seacrowd dataloader for id_vaccines_tweets, for every example in the dataset, it contains a subjective
    tweet and their sentiment polarity.  Tweets are from 20-27 June 2021 and 15-22 July 2021."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=_DESCRIPTION,
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=_DESCRIPTION,
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "idx": datasets.Value("string"),
                    "form_text": datasets.Value("string"),
                    "norm_text": datasets.Value("string"),
                    "subjective": datasets.Value("float"),
                    "sentiment": datasets.Value("float"),
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features([-1.0, 0.0, 1.0])
        else:
            raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """ "return splitGenerators"""
        downloaded_files = dl_manager.download_and_extract(_URL)
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files})]

    def _generate_examples(self, filepath):
        data_lines = pandas.read_csv(filepath, skip_blank_lines=True)
        keys = data_lines.keys()
        indexes = data_lines[keys[0]][1:]
        norms = data_lines[keys[1]][1:]
        formals = data_lines[keys[2]][1:]
        subjs = data_lines[keys[3]][1:]
        posnegs = data_lines[keys[4]][1:]
        if self.config.schema == "source":
            for idx, (ind, norm, form, subj, posneg) in enumerate(zip(indexes, norms, formals, subjs, posnegs)):
                yield idx, {
                    "idx": str(ind),
                    "form_text": form,
                    "norm_text": norm,
                    "subjective": float(subj),
                    "sentiment": float(posneg),
                }
        if self.config.schema == "seacrowd_text":
            for idx, (ind, norm, posneg) in enumerate(zip(indexes, norms, posnegs)):
                yield idx, {"id": str(ind), "text": norm, "label": float(posneg)}



import os
from typing import Dict, List, Tuple

try:
    from typing import Literal, TypedDict
except ImportError:
    from typing_extensions import Literal, TypedDict

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{id_panl_bppt,
  author    = {PAN Localization - BPPT},
  title     = {Parallel Text Corpora, English Indonesian},
  year      = {2009},
  url       = {http://digilib.bppt.go.id/sampul/p92-budiono.pdf},
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "id_panl_bppt"
_DESCRIPTION = """\
Parallel Text Corpora for Multi-Domain Translation System created by BPPT (Indonesian Agency for the Assessment and
Application of Technology) for PAN Localization Project (A Regional Initiative to Develop Local Language Computing
Capacity in Asia). The dataset contains about 24K sentences in English and Bahasa Indonesia from 4 different topics
(Economy, International Affairs, Science & Technology, and Sports).
"""
_HOMEPAGE = "http://digilib.bppt.go.id/sampul/p92-budiono.pdf"
_LICENSE = ""
_URLS = {
    _DATASETNAME: "https://github.com/cahya-wirawan/indonesian-language-models/raw/master/data/BPPTIndToEngCorpusHalfM.zip",
}
_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
# Source has no versioning
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IdPanlBppt(datasets.GeneratorBasedBuilder):
    """\
    Dataset contains about ~24K sentences in English and Bahasa Indonesia from 4 different topics (Economy,
    International Affairs, Science & Technology, and Sports)
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    class Topic(TypedDict):
        name: Literal["Economy", "International", "Science", "Sport"]
        # seems to be the number of words in the file
        words: Literal["150K", "100K"]

    TOPICS: List[Topic] = [{"name": "Economy", "words": "150K"}, {"name": "International", "words": "150K"}, {"name": "Science", "words": "100K"}, {"name": "Sport", "words": "100K"}]

    SOURCE_LANGUAGE = "en"
    TARGET_LANGUAGE = "id"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_panl_bppt_source",
            version=SOURCE_VERSION,
            description="PANL BPPT source schema",
            schema="source",
            subset_id="id_panl_bppt",
        ),
        SEACrowdConfig(
            name="id_panl_bppt_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="PANL BPPT Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="id_panl_bppt",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_panl_bppt_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "translation": datasets.features.Translation(languages=[self.SOURCE_LANGUAGE, self.TARGET_LANGUAGE]),
                    "topic": datasets.features.ClassLabel(names=list(map(lambda topic: topic["name"], self.TOPICS))),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "dir": os.path.join(data_dir, "plain"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, dir: str, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        id = 0
        for topic in self.TOPICS:
            src_path = f"PANL-BPPT-{topic['name'][:3].upper()}-{self.SOURCE_LANGUAGE.upper()}-{topic['words']}w.txt"
            tgt_path = f"PANL-BPPT-{topic['name'][:3].upper()}-{self.TARGET_LANGUAGE.upper()}-{topic['words']}w.txt"
            with open(os.path.join(dir, src_path), encoding="utf-8") as f1, open(os.path.join(dir, tgt_path), encoding="utf-8") as f2:
                src = f1.read().split("\n")[:-1]
                tgt = f2.read().split("\n")[:-1]
                for s, t in zip(src, tgt):
                    if self.config.schema == "source":
                        yield id, {
                            "id": str(id),
                            "translation": {self.SOURCE_LANGUAGE: s, self.TARGET_LANGUAGE: t},
                            "topic": topic["name"],
                        }
                    elif self.config.schema == "seacrowd_t2t":
                        # Schema does not have topics or any other fields to have the topics
                        yield id, {
                            "id": str(id),
                            "text_1": s,
                            "text_2": t,
                            "text_1_name": self.SOURCE_LANGUAGE,
                            "text_2_name": self.TARGET_LANGUAGE,
                        }

                    id += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
IndoNLI is the first human-elicited Natural Language Inference (NLI) dataset for Indonesian.
IndoNLI is annotated by both crowd workers and experts. The expert-annotated data is used exclusively as a test set.
It is designed to provide a challenging test-bed for Indonesian NLI by explicitly incorporating various linguistic
phenomena such as numerical reasoning, structural changes, idioms, or temporal and spatial reasoning.

The data is split across train, valid, test_lay, and test_expert.

A small subset of test_expert is used as a diasnostic tool. For more info, please visit https://github.com/ir-nlp-csui/indonli

The premise were collected from Indonesian Wikipedia and from other public Indonesian dataset: Indonesian PUD and GSD treebanks provided by the Universal Dependencies 2.5 and IndoSum

The data was produced by humans.

"""

from pathlib import Path
from typing import List

import datasets
import jsonlines

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{mahendra-etal-2021-indonli,
    title = "{I}ndo{NLI}: A Natural Language Inference Dataset for {I}ndonesian",
    author = "Mahendra, Rahmad and Aji, Alham Fikri and Louvan, Samuel and Rahman, Fahrurrozi and Vania, Clara",
    booktitle = "Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing",
    month = nov,
    year = "2021",
    address = "Online and Punta Cana, Dominican Republic",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.emnlp-main.821",
    pages = "10511--10527",
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "indonli"

_DESCRIPTION = """\
This dataset is designed for Natural Language Inference NLP task.  It is designed to provide a challenging test-bed
for Indonesian NLI by explicitly incorporating various linguistic phenomena such as numerical reasoning, structural
changes, idioms, or temporal and spatial reasoning.
"""

_HOMEPAGE = "https://github.com/ir-nlp-csui/indonli"

_LICENSE = "Creative Common Attribution Share-Alike 4.0 International"

# For publicly available datasets you will most likely end up passing these URLs to dl_manager in _split_generators.
# In most cases the URLs will be the same for the source and seacrowd config.
# However, if you need to access different files for each config you can have multiple entries in this dict.
# This can be an arbitrarily nested dict/list of URLs (see below in `_split_generators` method)
_URLS = {
    _DATASETNAME: {
        "train": "https://raw.githubusercontent.com/ir-nlp-csui/indonli/main/data/indonli/train.jsonl",
        "valid": "https://raw.githubusercontent.com/ir-nlp-csui/indonli/main/data/indonli/val.jsonl",
        "test": "https://raw.githubusercontent.com/ir-nlp-csui/indonli/main/data/indonli/test.jsonl",
    }
}

_SUPPORTED_TASKS = [Tasks.TEXTUAL_ENTAILMENT]

_SOURCE_VERSION = "1.1.0"  # Mentioned in https://github.com/huggingface/datasets/blob/main/datasets/indonli/indonli.py

_SEACROWD_VERSION = "1.0.0"


class IndoNli(datasets.GeneratorBasedBuilder):
    """IndoNLI, a human-elicited NLI dataset for Indonesian containing ~18k sentence pairs annotated by crowd workers."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indonli_source",
            version=SOURCE_VERSION,
            description="indonli source schema",
            schema="source",
            subset_id="indonli",
        ),
        SEACrowdConfig(
            name="indonli_seacrowd_pairs",
            version=SEACROWD_VERSION,
            description="indonli Nusantara schema",
            schema="seacrowd_pairs",
            subset_id="indonli",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indonli_source"
    labels = ["c", "e", "n"]

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "pair_id": datasets.Value("int32"),
                    "premise_id": datasets.Value("int32"),
                    "premise": datasets.Value("string"),
                    "hypothesis": datasets.Value("string"),
                    "annotator_type": datasets.Value("string"),
                    "sentence_size": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_pairs":
            features = schemas.pairs_features(self.labels)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        train_data_path = Path(dl_manager.download_and_extract(urls["train"]))
        valid_data_path = Path(dl_manager.download_and_extract(urls["valid"]))
        test_data_path = Path(dl_manager.download_and_extract(urls["test"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": train_data_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": valid_data_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": test_data_path},
            ),
        ]

    def _generate_examples(self, filepath: Path):

        if self.config.schema == "source":
            print(filepath)
            with jsonlines.open(filepath) as f:
                skip = []  # To avoid duplicate IDs
                for example in f.iter():
                    if example["pair_id"] not in skip:
                        skip.append(example["pair_id"])
                        example = {
                            "pair_id": example["pair_id"],
                            "premise_id": example["premise_id"],
                            "premise": example["premise"],
                            "hypothesis": example["hypothesis"],
                            "annotator_type": example["annotator_type"],
                            "sentence_size": example["sentence_size"],
                            "label": example["label"],
                        }
                        yield example["pair_id"], example

        elif self.config.schema == "seacrowd_pairs":
            print(filepath)
            with jsonlines.open(filepath) as f:
                skip = []  # To avoid duplicate IDs
                for example in f.iter():
                    if example["pair_id"] not in skip:
                        skip.append(example["pair_id"])
                        nu_eg = {"id": str(example["pair_id"]), "text_1": example["premise"], "text_2": example["hypothesis"], "label": example["label"]}
                        yield example["pair_id"], nu_eg
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from pathlib import Path
from typing import List

import datasets
import pandas as pd
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

# No paper citation found.
_CITATION = ""

_LOCAL = False
_LANGUAGES = ["tha"]
_DATASETNAME = "thai_alpaca"
_DESCRIPTION = """\
This is a Thai 🇹🇭-instructed dataset translated from cleaned version of the original
Alpaca Dataset released by Stanford using Google Cloud Translation, contain 52,000
instructions and demonstrations generated by OpenAI's text-davinci-003 engine. This
instruction data can be used to conduct instruction-tuning for language models and
make the language model follow instruction better.
"""

_HOMEPAGE = "https://huggingface.co/datasets/Thaweewat/alpaca-cleaned-52k-th"
_LICENSE = Licenses.CC_BY_NC_4_0.value
_URL = "https://huggingface.co/datasets/Thaweewat/alpaca-cleaned-52k-th/resolve/main/alpaca-cleaned-th.parquet"
_SUPPORTED_TASKS = [Tasks.INSTRUCTION_TUNING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ThaiAlpacaDataset(datasets.GeneratorBasedBuilder):
    """Thai Alpaca Dataset"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "t2t"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description="Thai-Alpaca source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description="Thai-Alpaca SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = "thai_alpaca_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "instruction": datasets.Value("string"),
                    "input": datasets.Value("string"),
                    "output": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_file = Path(dl_manager.download_and_extract(_URL))
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": data_file})]

    def _generate_examples(self, filepath: Path):
        """Yield examples as (key, example) tuples"""
        df = pd.read_parquet(filepath)
        for idx, row in df.iterrows():
            if self.config.schema == "source":
                example = {"instruction": row.get("instruction"), "input": row.get("input"), "output": row.get("output")}

            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                inputs = row.get("input")
                if inputs:
                    text_1 = f"Context: {inputs}\n\n{row.get('instruction')}"
                else:
                    text_1 = f"Context: {row.get('instruction')}"

                example = {
                    "id": str(idx),
                    "text_1": text_1,
                    "text_2": row.get("output"),
                    "text_1_name": "input_instruction",
                    "text_2_name": "output",
                }

            yield idx, example



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import jsonlines
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{,
  author    = {supryzhu},
  title     = {Indonesia-Chinese-MTRobustEval},
  journal   = {None},
  volume    = {None},
  year      = {2023},
  url       = {https://github.com/supryzhu/Indonesia-Chinese-MTRobustEval},
  doi       = {None},
  biburl    = {None},
  bibsource = {None}
}
"""


_DATASETNAME = "indonesia_chinese_mtrobusteval"

_DESCRIPTION = """\
The dataset is curated for the purpose of evaluating the robustness of Neural Machine Translation (NMT) towards natural occuring noise
(typo, slang, code switching, etc.). The dataset is crawled from Twitter, then pre-processed to obtain sentences with noise.
The dataset consists of a thousand noisy sentences. The dataset is translated into Chinese manually as the benchmark for evaluating the robustness of NMT.
"""

_HOMEPAGE = "https://github.com/supryzhu/Indonesia-Chinese-MTRobustEval"

_LANGUAGES = ["ind", "cmn"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)


_LICENSE = Licenses.MIT.value  # example: Licenses.MIT.value, Licenses.CC_BY_NC_SA_4_0.value, Licenses.UNLICENSE.value, Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/supryzhu/Indonesia-Chinese-MTRobustEval/raw/main/data/Indonesia-Chinese.xlsx",
}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IndonesiaChineseMtRobustEval(datasets.GeneratorBasedBuilder):
    """The dataset consists of a thousand noisy sentences. The dataset is translated into Chinese manually as the benchmark for evaluating the robustness of NMT."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description="indonesia_chinese_mtrobusteval source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="indonesia_chinese_mtrobusteval SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "src": datasets.Value("string"),
                    "tgt": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        file_path = dl_manager.download(urls)
        df = pd.read_excel(file_path)
        src = df["Indonesia"].tolist()
        tgt = df["Chinese"].tolist()
        results = []
        for i, item in enumerate(src):
            results.append({"id": str(i), "src": item, "tgt": tgt[i]})
        self._write_jsonl(file_path + ".jsonl", results)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": file_path + ".jsonl",
                    "split": "train",
                },
            )
        ]


    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        if self.config.schema == "source":
            i = 0
            with jsonlines.open(filepath) as f:
                for each_data in f.iter():
                    ex = {
                        "id": each_data["id"],
                        "src": each_data["src"],
                        "tgt": each_data["tgt"],
                    }
                    yield i, ex
                    i += 1

        elif self.config.schema == "seacrowd_t2t":
            i = 0
            with jsonlines.open(filepath) as f:
                for each_data in f.iter():
                    ex = {"id": each_data["id"], "text_1": each_data["src"], "text_2": each_data["tgt"], "text_1_name": "ind", "text_2_name": "cmn"}
                    yield i, ex
                    i += 1

    def _write_jsonl(self, filepath, values):
        with jsonlines.open(filepath, "w") as writer:
            for line in values:
                writer.write(line)




# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import re
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{limkonchotiwat-etal-2021-handling,
    title = "Handling Cross- and Out-of-Domain Samples in {T}hai Word Segmentation",
    author = "Limkonchotiwat, Peerat  and
      Phatthiyaphaibun, Wannaphong  and
      Sarwar, Raheem  and
      Chuangsuwanich, Ekapol  and
      Nutanong, Sarana",
    booktitle = "Findings of the Association for Computational Linguistics: ACL-IJCNLP 2021",
    month = aug,
    year = "2021",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.findings-acl.86",
    doi = "10.18653/v1/2021.findings-acl.86",
    pages = "1003--1016",
}
"""

_DATASETNAME = "vistec_tp_th_21"

_DESCRIPTION = """\
The largest social media domain datasets for Thai text processing (word segmentation, 
misspell correction and detection, and named-entity boundary) called "VISTEC-TP-TH-2021" or VISTEC-2021. 
VISTEC corpus contains 49,997 sentences with 3.39M words where the collection was manually annotated by 
linguists on four tasks, namely word segmentation, misspelling detection and correction, 
and named entity recognition.
"""

_HOMEPAGE = "https://github.com/mrpeerat/OSKut/tree/main/VISTEC-TP-TH-2021"


_LANGUAGES = ["tha"]


_LICENSE = Licenses.CC_BY_SA_3_0.value

_LOCAL = False

_URLS = {
    "train": "https://raw.githubusercontent.com/mrpeerat/OSKut/main/VISTEC-TP-TH-2021/train/VISTEC-TP-TH-2021_train_proprocessed.txt",
    "test": "https://raw.githubusercontent.com/mrpeerat/OSKut/main/VISTEC-TP-TH-2021/test/VISTEC-TP-TH-2021_test_proprocessed.txt",
}

_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class VISTEC21Dataset(datasets.GeneratorBasedBuilder):
    """
    The largest social media domain datasets for Thai text processing (word segmentation,
    misspell correction and detection, and named-entity boundary) called "VISTEC-TP-TH-2021" or VISTEC-2021.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "seq_label"
    LABEL_CLASSES = ["0", "1"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(datasets.features.ClassLabel(names=self.LABEL_CLASSES)),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.seq_label_features(self.LABEL_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_files = {
            "train": Path(dl_manager.download_and_extract(_URLS["train"])),
            "test": Path(dl_manager.download_and_extract(_URLS["test"])),
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"], "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"], "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        label_key = "ner_tags" if self.config.schema == "source" else "labels"

        with open(filepath, "r", encoding="utf-8") as f:
            lines = f.readlines()
            id = 0
            for line in lines:
                tokens = line.split("|")
                token_list = []
                ner_tag = []
                for token in tokens:
                    if "<ne>" in token:
                        token = token.replace("<ne>", "")
                        token = token.replace("</ne>", "")
                        token_list.append(token)
                        ner_tag.append(1)
                        continue
                    if "</msp>" in token and "<msp value=" in token:
                        token_list.append(re.findall(r"<msp value=([^>]*)>", token)[0])
                        ner_tag.append(0)
                        continue
                    if "<compound>" in token or "</compound>" in token:
                        token = token.replace("<compound>", "")
                        token = token.replace("</compound>", "")
                        token_list.append(token)
                        ner_tag.append(0)
                        continue
                    token_list.append(token)
                    ner_tag.append(0)
                id += 1
                yield id, {
                    "id": str(id),
                    "tokens": token_list,
                    label_key: ner_tag,
                }



import os
from pathlib import Path

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@article{cruz2019evaluating,
  title={Evaluating Language Model Finetuning Techniques for Low-resource Languages},
  author={Cruz, Jan Christian Blaise and Cheng, Charibeth},
  journal={arXiv preprint arXiv:1907.00409},
  year={2019}
}
"""

_DATASETNAME = "wikitext_tl_39"

_DESCRIPTION = """A benchmark Language Modeling dataset for Tagalog. The dataset construction was done similar to that of the WikiText
Long Term Dependency Language Modeling Dataset, with a some differences, such as in how Wikipedia was scraped and how the vocabulary was
created. The dataset contains 39 Million tokens in the training set.
"""

_HOMEPAGE = "https://huggingface.co/datasets/wikitext_tl39"

_LANGUAGES = ["fil"]

_LICENSE = Licenses.GPL_3_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://s3.us-east-2.amazonaws.com/blaisecruz.com/datasets/wikitext-tl-39/wikitext-tl-39.zip",
}

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class WikiTextTL39Dataset(datasets.GeneratorBasedBuilder):
    """Large scale, unlabeled text dataset with 39 Million tokens in the training set in Tagalog."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_ssp",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_ssp",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        features = schemas.ssp_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> list[datasets.SplitGenerator]:
        data_dir = dl_manager.download_and_extract(_URLS[_DATASETNAME])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(data_dir, "wikitext-tl-39", "train.txt"), "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, "wikitext-tl-39", "test.txt"), "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(data_dir, "wikitext-tl-39", "valid.txt"), "split": "valid"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> tuple[int, dict]:
        with open(filepath, encoding="utf-8") as f:
            for i, row in enumerate(f):
                if row.strip():
                    yield i, {
                        "id": str(i),
                        "text": row,
                    }
                else:
                    yield i, {
                        "id": str(i),
                        "text": "",
                    }



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@incollection{nguyen2021vietnamese,
  title={Vietnamese Complaint Detection on E-Commerce Websites},
  author={Nguyen, Nhung Thi-Hong and Ha, Phuong Phan-Dieu and Nguyen, Luan Thanh and Nguyen, Kiet Van and Nguyen, Ngan Luu-Thuy},
  booktitle={New Trends in Intelligent Software Methodologies, Tools and Techniques},
  pages={618--629},
  year={2021},
  publisher={IOS Press}
}
"""

_DATASETNAME = "uit_viocd"

_DESCRIPTION = """\
The UIT-ViOCD dataset includes 5,485 reviews e-commerce sites across four categories: fashion, cosmetics, applications,
and phones. Each review is annotated by humans, assigning a label of 1 for complaints and 0 for non-complaints.
The dataset is divided into training, validation, and test sets, distributed approximately in an 80:10:10 ratio.
"""

_HOMEPAGE = "https://huggingface.co/datasets/tarudesu/ViOCD"

_LANGUAGES = ["vie"]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    "train": "https://huggingface.co/datasets/tarudesu/ViOCD/resolve/main/train.csv?download=true",
    "val": "https://huggingface.co/datasets/tarudesu/ViOCD/resolve/main/val.csv?download=true",
    "test": "https://huggingface.co/datasets/tarudesu/ViOCD/resolve/main/test.csv?download=true",
}

_SUPPORTED_TASKS = [Tasks.COMPLAINT_DETECTION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class UITVIOCDDataset(datasets.GeneratorBasedBuilder):
    """The UIT-ViOCD dataset includes 5,485 reviews e-commerce sites across four categories: fashion, cosmetics, applications, and phones."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    LABEL_CLASSES = [1, 0]

    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "review": datasets.Value("string"),
                    "review_tokenize": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=self.LABEL_CLASSES),
                    "domain": datasets.Value("string"),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(self.LABEL_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        data_dir = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["val"],
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        df = pd.read_csv(filepath)

        if self.config.schema == "source":
            for key, example in df.iterrows():
                yield key, {
                    "review": example["review"],
                    "review_tokenize": example["review_tokenize"],
                    "label": example["label"],
                    "domain": example["domain"],
                }

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for key, example in df.iterrows():
                yield key, {"id": str(key), "text": str(example["review"]), "label": int(example["label"])}



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{Chrismanto2022,
title = {SPAMID-PAIR: A Novel Indonesian Post–Comment Pairs Dataset Containing Emoji},
journal = {International Journal of Advanced Computer Science and Applications},
doi = {10.14569/IJACSA.2022.0131110},
url = {http://dx.doi.org/10.14569/IJACSA.2022.0131110},
year = {2022},
publisher = {The Science and Information Organization},
volume = {13},
number = {11},
author = {Antonius Rachmat Chrismanto and Anny Kartika Sari and Yohanes Suyanto}
}
"""

_DATASETNAME = "spamid_pair"


_DESCRIPTION = """\
SPAMID-PAIR is data post-comment pairs collected from 13 selected Indonesian public figures (artists) / public accounts
with more than 15 million followers and categorized as famous artists.
It was collected from Instagram using an online tool and Selenium.
Two persons labeled all pair data as an expert in a total of 72874 data.
The data contains Unicode text (UTF-8) and emojis scrapped in posts and comments without account profile information.
"""

_HOMEPAGE = "https://data.mendeley.com/datasets/fj5pbdf95t/1"

_LANGUAGES = ["ind"]


_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False


_URLS = {
    _DATASETNAME: "https://prod-dcd-datasets-cache-zipfiles.s3.eu-west-1.amazonaws.com/fj5pbdf95t-1.zip",
}

_SUPPORTED_TASKS = [Tasks.INTENT_CLASSIFICATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class SpamidPairDataset(datasets.GeneratorBasedBuilder):
    """SPAMID-PAIR is data post-comment pairs collected from 13 selected Indonesian public figures (artists) / public accounts with more than 15 million followers and categorized as famous artists."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    LABEL_CLASSES = [1, 0]

    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "igid": datasets.Value("string"),
                    "comment": datasets.Value("string"),
                    "posting": datasets.Value("string"),
                    "spam": datasets.ClassLabel(names=self.LABEL_CLASSES),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(self.LABEL_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = Path(dl_manager.download_and_extract(urls))
        data_dir = os.path.join(os.path.join(os.path.join(data_dir, "SPAMID-PAIR"), "Raw"), "dataset-raw.xlsx")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        data = pd.read_excel(filepath)

        if self.config.schema == "source":
            for i, row in data.iterrows():
                yield i, {
                    "igid": str(row["igid"]),
                    "comment": str(row["comment"]),
                    "posting": str(row["posting"]),
                    "spam": row["spam"],
                }

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for i, row in data.iterrows():
                yield i, {
                    "id": str(i),
                    "text": str(row["comment"]) + "\n" + str(row["posting"]),
                    "label": int(row["spam"]),
                }



import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@inproceedings{koto-etal-2023-large,
    title = "Large Language Models Only Pass Primary School Exams in {I}ndonesia: A Comprehensive Test on {I}ndo{MMLU}",
    author = "Koto, Fajri  and
      Aisyah, Nurul  and
      Li, Haonan  and
      Baldwin, Timothy",
    editor = "Bouamor, Houda  and
      Pino, Juan  and
      Bali, Kalika",
    booktitle = "Proceedings of the 2023 Conference on Empirical Methods in Natural Language Processing",
    month = dec,
    year = "2023",
    address = "Singapore",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2023.emnlp-main.760",
    doi = "10.18653/v1/2023.emnlp-main.760",
    pages = "12359--12374",
}
"""

_DATASETNAME = "indommlu"

_DESCRIPTION = """
IndoMMLU is the first multi-task language understanding benchmark for Indonesian culture and languages, which consists
of questions from primary school to university entrance exams in Indonesia. By employing professional teachers, we
obtain 14,906 questions across 63 tasks and education levels, with 46% of the questions focusing on assessing
proficiency in the Indonesian language and knowledge of nine local languages and cultures in Indonesia.
"""

_HOMEPAGE = "https://huggingface.co/datasets/indolem/IndoMMLU"

_LANGUAGES = ["ind", "ban", "mad", "nij", "sun", "jav", "mak", "bjn", "abl"]

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value

_LOCAL = False

_URLS = {_DATASETNAME: {"test": "https://huggingface.co/datasets/indolem/IndoMMLU/resolve/main/IndoMMLU.csv"}}

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


lang2subject = {"ind": "Bahasa Indonesia", "ban": "Bahasa Bali", "mad": "Bahasa Madura", "nij": "Bahasa Dayak Ngaju", "sun": "Bahasa Sunda", "jav": "Bahasa Jawa", "mak": "Bahasa Makassar", "bjn": "Bahasa Banjar", "abl": "Bahasa Lampung"}

subject2english = {
    "Sejarah": "History",
    "Geografi": "Geography",
    "Bahasa Lampung": "Lampungic",
    "IPS": "Social science",
    "Bahasa Bali": "Balinese",
    "Bahasa Makassar": "Makassarese",
    "Bahasa Banjar": "Banjarese",
    "Kimia": "Chemistry",
    "Biologi": "Biology",
    "IPA": "Science",
    "Agama Kristen": "Christian religion",
    "Kesenian": "Art",
    "Agama Islam": "Islam religion",
    "Agama Hindu": "Hindu religion",
    "Bahasa Madura": "Madurese",
    "Penjaskes": "Sport",
    "Bahasa Indonesia": "Indonesian language",
    "Fisika": "Physics",
    "Budaya Alam Minangkabau": "Minangkabau culture",
    "Bahasa Dayak Ngaju": "Dayak language",
    "Sosiologi": "Sociology",
    "Ekonomi": "Economy",
    "Bahasa Sunda": "Sundanese",
    "Bahasa Jawa": "Javanese",
    "PPKN": "Civic education",
}

subject2group = {
    "Sejarah": "Humanities",
    "Geografi": "Social science",
    "Bahasa Lampung": "Local languages and cultures",
    "IPS": "Social science",
    "Bahasa Bali": "Local languages and cultures",
    "Bahasa Makassar": "Local languages and cultures",
    "Bahasa Banjar": "Local languages and cultures",
    "Kimia": "STEM",
    "Biologi": "STEM",
    "IPA": "STEM",
    "Agama Kristen": "Humanities",
    "Kesenian": "Humanities",
    "Agama Islam": "Humanities",
    "Agama Hindu": "Humanities",
    "Bahasa Madura": "Local languages and cultures",
    "Penjaskes": "Humanities",
    "Bahasa Indonesia": "Indonesian language",
    "Fisika": "STEM",
    "Budaya Alam Minangkabau": "Local languages and cultures",
    "Bahasa Dayak Ngaju": "Local languages and cultures",
    "Sosiologi": "Social science",
    "Ekonomi": "Social science",
    "Bahasa Sunda": "Local languages and cultures",
    "Bahasa Jawa": "Local languages and cultures",
    "PPKN": "Social science",
}

special_case = ["SD-SMP-SMA", "SD-SMP"]
level_mapper = {
    "SMA": "SMA",  # SMA --> high school level"
    "Seleksi PTN": "University entrance test",
    "SD": "SD",  # SD --> elementary school level
    "SMP": "SMP",  # SMP --> junior high school level
    "Kelas I SD": "SD",
    "Kelas X SMA": "SMA",
    "Kelas XI SMA": "SMA",
    "Kelas XII SMA": "SMA",
    "V SD": "SD",
    "VI SD": "SD",
    "VII SMP": "SMP",
    "VIII SMP ": "SMP",
    "IX SMP": "SMP",
    "Kelas III SD": "SD",
    "Kelas IV SD": "SD",
    "Kelas II SD": "SD",
}


def fix_level(level, kelas):
    # Fixing Level
    if level in special_case:
        kelas = float(kelas)
        if kelas >= 1 and kelas <= 6:
            level = "SD"
        elif kelas >= 7 and kelas <= 9:
            level = "SMP"
        elif kelas >= 10:
            level = "SMA"
        else:
            print(level)
    fixed_level = level_mapper[level]

    # Fixing class
    kelas = str(kelas)
    if kelas.strip() in ["PTN", "2023-10-12 00:00:00"]:
        fixed_kelas = 13
    elif kelas == "4,5,6":
        fixed_kelas = 6
    else:
        fixed_kelas = int(float(kelas.strip()))

    # sanity check over the level and kelas
    return fixed_level, fixed_kelas


def pass_schema_filter(schema, row):
    if schema == "source":
        return True
    lang = schema.split("_")[1]
    if lang not in _LANGUAGES:  # seacrowd_qa
        return True
    if lang == "ind":  # contains "Bahasa Indonesia" and all other non-language subjects
        return (lang2subject[lang] == row["subject"]) or (row["subject"] not in lang2subject.values())
    return lang2subject[lang] == row["subject"]


class IndoMMLUDataset(datasets.GeneratorBasedBuilder):
    """IndoMMLU is the first multitask language understanding benchmark for Indonesian culture and languages."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_qa",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_qa",
            subset_id=_DATASETNAME,
        ),
    ]
    for lang in _LANGUAGES:
        lang_config = SEACrowdConfig(
            name=f"{_DATASETNAME}_{lang}_seacrowd_qa",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} {lang} SEACrowd schema",
            schema=f"seacrowd_qa",
            subset_id=_DATASETNAME,
        )
        BUILDER_CONFIGS.append(lang_config)

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "subject": datasets.Value("string"),
                    "group": datasets.Value("string"),
                    "level": datasets.Value("string"),
                    "class": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "options": datasets.Value("string"),
                    "answer": datasets.Value("string"),
                    "is_for_fewshot": datasets.Value("string"),
                }
            )

        else:
            features = schemas.qa_features
            features["meta"] = {
                "subject": datasets.Value("string"),
                "group": datasets.Value("string"),
                "level": datasets.Value("string"),
                "class": datasets.Value("string"),
                "is_for_fewshot": datasets.Value("string"),
            }

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_dir, "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        data = csv.DictReader(open(filepath[split], newline=""))
        print(self.config.schema)
        for i, row in enumerate(data):
            if pass_schema_filter(self.config.schema, row):
                fixed_level, fixed_kelas = fix_level(row["level"], row["kelas"])
                # The choices are in the format of ["A. xxx", "B. xxx", ...], but answer is only with ["A"], replacing both with only the answer content
                choices = row["jawaban"].split("\n")
                answer_choice = row["kunci"]
                # Find the corresponding choice in the choices.
                # Skip the 2 datapoint (i = 4223, 14150) with invalid answer_choice.
                corresponding_choice = next((choice for choice in choices if choice.startswith(answer_choice)), None)
                if corresponding_choice is None:
                    continue
                else:
                    if self.config.schema == "source":
                        yield i, {
                            "subject": subject2english[row["subject"]],
                            "group": subject2group[row["subject"]],
                            "level": fixed_level,
                            "class": fixed_kelas,
                            "question": row["soal"],
                            "options": [opt[2:].strip() for opt in choices],  # remove A., B., ... in the options,
                            "answer": corresponding_choice[2:].strip(),  # remove A., B., ... in the answer
                            "is_for_fewshot": row["is_for_fewshot"],
                        }
                    else:
                        yield i, {
                            "id": str(i),
                            "question_id": str(i),
                            "document_id": str(i),
                            "question": row["soal"],
                            "type": "multiple_choice",
                            "choices": [opt[2:].strip() for opt in choices],  # remove A., B., ... in the options
                            "context": "",
                            "answer": [corresponding_choice[2:].strip()],  # remove A., B., ... in the answer,
                            "meta": {"subject": subject2english[row["subject"]], "group": subject2group[row["subject"]], "level": fixed_level, "class": fixed_kelas, "is_for_fewshot": row["is_for_fewshot"]},
                        }



import csv
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

# no BibTeX citation
_CITATION = ""

_DATASETNAME = "wongnai_reviews"

_DESCRIPTION = """
Wongnai features over 200,000 restaurants, beauty salons, and spas across Thailand on its platform, with detailed
information about each merchant and user reviews. Its over two million registered users can search for what’s top rated
in Bangkok, follow their friends, upload photos, and do quick write-ups about the places they visit. Each write-up
(review) also comes with a rating score ranging from 1-5 stars. The task here is to create a rating prediction model
using only textual information.
"""

_HOMEPAGE = "https://huggingface.co/datasets/wongnai_reviews"

_LANGUAGES = ["tha"]

_LICENSE = Licenses.LGPL_3_0.value

_LOCAL = False

_URLS = {_DATASETNAME: "https://archive.org/download/wongnai_reviews/wongnai_reviews_withtest.zip"}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_CLASSES = ["1", "2", "3", "4", "5"]


class WongnaiReviewsDataset(datasets.GeneratorBasedBuilder):
    """WongnaiReviews consists reviews for over 200,000 restaurants, beauty salons, and spas across Thailand."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_text",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "review_body": datasets.Value("string"),
                    "star_rating": datasets.ClassLabel(names=_CLASSES),
                }
            )

        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(label_names=_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(data_dir, "w_review_train.csv"), "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, "w_review_test.csv"), "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        if self.config.schema == "source":
            with open(filepath, encoding="utf-8") as f:
                spamreader = csv.reader(f, delimiter=";", quotechar='"')
                for i, row in enumerate(spamreader):
                    yield i, {"review_body": row[0], "star_rating": row[1]}

        elif self.config.schema == "seacrowd_text":
            with open(filepath, encoding="utf-8") as f:
                spamreader = csv.reader(f, delimiter=";", quotechar='"')
                for i, row in enumerate(spamreader):
                    yield i, {"id": str(i), "text": row[0], "label": _CLASSES[int(row[1].strip()) - 1]}



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from conllu import TokenList

from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_ud_data, load_ud_data_as_seacrowd_kb
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@article {10.3844/jcssp.2020.1585.1597,
author = {Alfina, Ika and Budi, Indra and Suhartanto, Heru},
title = {Tree Rotations for Dependency Trees: Converting the Head-Directionality of Noun Phrases},
article_type = {journal},
volume = {16},
number = {11},
year = {2020},
month = {Nov},
pages = {1585-1597},
doi = {10.3844/jcssp.2020.1585.1597},
url = {https://thescipub.com/abstract/jcssp.2020.1585.1597},
journal = {Journal of Computer Science},
publisher = {Science Publications}
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "ud_id_csui"

_DESCRIPTION = """\
UD Indonesian-CSUI is a conversion from an Indonesian constituency treebank in the Penn Treebank format named Kethu that was also a conversion from a constituency treebank built by Dinakaramani et al. (2015).
This treebank is named after the place where treebanks were built: Faculty of Computer Science (CS), Universitas Indonesia (UI).

About this treebank:
- Genre is news in formal Indonesian (the majority is economic news)
- 1030 sentences (28K words) divided into testing and training dataset of around 10K words and around 18K words respectively.
- Average of 27.4 words per-sentence.
"""

_HOMEPAGE = "https://github.com/UniversalDependencies/UD_Indonesian-CSUI"

_LICENSE = "CC BY-SA 4.0"

_URLS = {
    _DATASETNAME: {
        "train": "https://raw.githubusercontent.com/UniversalDependencies/UD_Indonesian-CSUI/master/id_csui-ud-train.conllu",
        "test": "https://raw.githubusercontent.com/UniversalDependencies/UD_Indonesian-CSUI/master/id_csui-ud-test.conllu",
    },
}

_SUPPORTED_TASKS = [Tasks.DEPENDENCY_PARSING, Tasks.MACHINE_TRANSLATION, Tasks.POS_TAGGING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class UdIdCsuiDataset(datasets.GeneratorBasedBuilder):
    """Treebank of formal Indonesian news which consists of 1030 sentences (28K words)"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    # source: https://universaldependencies.org/u/pos/
    UPOS_TAGS = ["ADJ", "ADP", "ADV", "AUX", "CCONJ", "DET", "INTJ", "NOUN", "NUM", "PART", "PRON", "PROPN", "PUNCT", "SCONJ", "SYM", "VERB", "X"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_kb",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} Nusantara KB schema",
            schema="seacrowd_kb",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} Nusantara Text to Text schema",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} Nusantara Seq Label schema",
            schema="seacrowd_seq_label",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    # metadata
                    "sent_id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "text_en": datasets.Value("string"),
                    # tokens
                    "id": [datasets.Value("string")],
                    "form": [datasets.Value("string")],
                    "lemma": [datasets.Value("string")],
                    "upos": [datasets.Value("string")],
                    "xpos": [datasets.Value("string")],
                    "feats": [datasets.Value("string")],
                    "head": [datasets.Value("string")],
                    "deprel": [datasets.Value("string")],
                    "deps": [datasets.Value("string")],
                    "misc": [datasets.Value("string")],
                }
            )

        elif self.config.schema == "seacrowd_kb":
            features = schemas.kb_features

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.UPOS_TAGS)

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_path = dl_manager.download(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_path["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_path["test"],
                },
            ),
        ]

    @staticmethod
    def _assert_multispan_range_is_one(token_list: TokenList):
        """
        Asserting that all tokens with multiple span can only have 2 span, and \
        no field other than form has important information
        """
        for token in token_list.filter(id=lambda i: not isinstance(i, int)):
            _id = token["id"]
            assert len(_id) == 3, f"Unexpected length of non-int CONLLU Token's id. Expected 3, found {len(_id)};"
            assert all(isinstance(a, b) for a, b in zip(_id, [int, str, int])), f"Non-int ID should be in format of '\\d+-\\d+'. Found {_id};"
            assert _id[2] - _id[0] == 1, f"Token has more than 2 spans. Found {_id[2] - _id[0] + 1} spans;"
            for key in ["lemma", "upos", "xpos", "feats", "head", "deprel", "deps"]:
                assert token[key] in {"_", None}, f"Field other than 'form' should not contain extra information. Found: '{key}' = '{token[key]}'"

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # method parameters are unpacked from `gen_kwargs` as given in `_split_generators`

        dataset = list(load_ud_data(filepath, filter_kwargs={"id": lambda i: isinstance(i, int)}, assert_fn=self._assert_multispan_range_is_one))

        if self.config.schema == "source":
            pass

        elif self.config.schema == "seacrowd_kb":
            dataset = load_ud_data_as_seacrowd_kb(filepath, dataset)

        elif self.config.schema == "seacrowd_t2t":
            dataset = list(
                map(
                    lambda d: {
                        "id": d["sent_id"],
                        "text_1": d["text"],
                        "text_2": d["text_en"],
                        "text_1_name": "ind",
                        "text_2_name": "eng",
                    },
                    dataset,
                )
            )

        elif self.config.schema == "seacrowd_seq_label":
            dataset = list(
                map(
                    lambda d: {
                        "id": d["sent_id"],
                        "tokens": d["form"],
                        "labels": d["upos"],
                    },
                    dataset,
                )
            )

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        for key, example in enumerate(dataset):
            yield key, example


if __name__ == "__main__":
    datasets.load_dataset(__file__)
